import logging

class Logger:
    def __init__(self):
        self.logger = logging.getLogger("cleanuptotal")

    def debug(self, message: object):
        self.logger.debug(message)
    
    def info(self, message: object):
        self.logger.info(message)
    
    def error(self, message: object):
        self.logger.error(message)


import inspect
import traceback
from typing import Callable
from helpers.logger import Logger

class Cleanup:
    _cleanupList = []
    """This field is private, please don't change it.
    """
    logger = Logger()

    def __init__(self):
        self._cleanupList = []
        
    def add_cleanup(self, cleanupFunction: Callable) -> None:
        """Add cleanup function to the cleanup stack.

        Args:
            cleanupFunction (Callable): a cleanup function. It can be a lambda or regular function.
        """
        self._cleanupList.append(cleanupFunction)
    
      
    def finalize(self) -> None:
        """Please don't call this method directly.
        """
        errors = []
        if len(self._cleanupList) == 0:
            return

        self._cleanupList.reverse()
        
        for cleanup_function in self._cleanupList:
            try:
                source = inspect.getsource(cleanup_function)
                cleanup_function()
                self.logger.debug(f"pytest-cleanuptotal[😊]: Successfully executed '{source}' ({cleanup_function})")
            except Exception as err:
                message = f"pytest-cleanuptotal[😕]: Failed to execute '{source}' ({cleanup_function}): {err}, ${traceback.format_exc()}"
                errors.append(message)
            

        if len(errors) > 0:
            self.logger.error(f"pytest-cleanuptotal: Warning!!!: Cleanup finished with {len(errors)} error(s):")

            for run_error in errors:
                self.logger.error(run_error)
            

        self._cleanupList = []
        
        



from pytest_cleanuptotal.cleanup import Cleanup

def test_fail(cleanuptotal: Cleanup):
  cleanuptotal.add_cleanup(lambda: print("cleanup error stage 1"))

  def cleanup_function():
    print("before-cleanup error stage 2")
    raise Exception("Intentional exception")
    print("after-cleanup error stage 2")
  
  cleanuptotal.add_cleanup(cleanup_function)

  cleanuptotal.add_cleanup(lambda: print("cleanup error stage 3"))

  

def test_pass(cleanuptotal: Cleanup):
  cleanuptotal.add_cleanup(lambda: print("Cleaning pytest_playwright stage 1"))

  cleanuptotal.add_cleanup(lambda: print("Cleaning pytest_playwright stage 2"))

  cleanuptotal.add_cleanup(lambda: print("Cleaning pytest_playwright stage 3"))

  cleanuptotal.add_cleanup(lambda: print("Cleaning pytest_playwright stage 4"))
  


from setuptools import find_packages, setup

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='pytest-cleanuptotal',
    packages=find_packages(where='src', exclude=['*tests*']),
    package_dir={'': 'src'},
    version='0.3.7',
    author='Tzur Paldi',
    author_email='tzur.paldi@outlook.com',
    license="Apache-2.0",
    url="https://github.com/tzurp/pytest_cleanuptotal",
    project_urls={
        "Bug Tracker": "https://github.com/tzurp/pytest_cleanuptotal/issues",
        "Documentation": "https://github.com/tzurp/pytest_cleanuptotal#readme",
        "Source Code": "https://github.com/tzurp/pytest_cleanuptotal"
    },
    maintainer='Tzur Paldi',
    maintainer_email='tzur.paldi@outlook.com',
    description='A cleanup plugin for pytest',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Pytest",
        "License :: OSI Approved :: Apache Software License",
    ],
    entry_points={
        'pytest11': [
            'pytest_cleanuptotal = pytest_cleanuptotal.cleanuptotal',
        ],
    },
)



from typing import Generator
import pytest
from pytest_cleanuptotal.cleanup import Cleanup

@pytest.fixture
def cleanuptotal() -> Generator[Cleanup, None, None]:
    cleanup = Cleanup()
    yield cleanup
    cleanup.finalize()
